@@ -2,12 +2,17 @@ import urlparse
 import collections
 import re
 import urllib2
+import time
+import logging
+import base64
 
 from functools import wraps
 
 from monocle import _o, Return
 from monocle.stack.network import ConnectionLost, Client, SSLClient
 
+log = logging.getLogger(__name__)
+
 
 class HttpHeaders(collections.MutableMapping):
     def __init__(self, headers=None):
@@ -41,13 +46,17 @@ class HttpHeaders(collections.MutableMapping):
         return (x for x in self.headers)
 
     def __repr__(self):
-        return repr(self.headers)
+        return "<%s %s>" % (self.__class__.__name__, repr(self.headers))
 
-    def __getitem__(self, key):
+    def get_list(self, key):
         key = key.lower()
         if not key in self.keys:
             raise KeyError(key)
         vals = [v for k, v in self.headers if k == key]
+        return vals
+
+    def __getitem__(self, key):
+        vals = self.get_list(key)
         if len(vals) == 1:
             return vals[0]
         else:
@@ -68,6 +77,39 @@ class HttpHeaders(collections.MutableMapping):
         self.headers = [(k, v) for k, v in self.headers if k != key]
 
 
+class HttpRequest(object):
+    def __init__(self, proto='HTTP/1.0', host=None, method=None,
+                 uri=None, remote_ip=None, headers=None, body=None):
+        self.proto = proto
+        self.host = host
+        self.method = method
+        self.uri = uri
+        self.remote_ip = remote_ip
+        self.headers = headers
+        self.body = body
+
+        self.path, _, self.query = uri.partition('?')
+        self.arguments = urlparse.parse_qs(self.query, keep_blank_values=True)
+
+    def __repr__(self):
+        return "<%s (%s %s %s, headers=%s)>" % (
+            self.__class__.__name__, self.method, self.path, self.proto, self.headers)
+
+    def get_basic_auth(self):
+        if not "authorization" in self.headers:
+            return None, None
+        auth = self.headers["authorization"]
+        try:
+            method, b64 = auth.split(" ")
+            if method.lower() != "basic":
+                return None, None
+            username, password = base64.decodestring(b64).split(':', 1)
+        except Exception:
+            # parsing error; no valid auth
+            return None, None
+        return username, password
+
+
 class HttpResponse(object):
     def __init__(self, code, msg=None, headers=None, body=None, proto=None):
         self.code = code
@@ -304,7 +346,7 @@ class HttpRouter(object):
             return False, None
         return True, m.groupdict()
 
-    def mk_decorator(self, method, pattern):
+    def mk_decorator(self, method, pattern, add_head=False):
         if not hasattr(pattern, 'match'):
             pattern = re.escape(pattern)
             pattern = pattern.replace(r'\?', '?')
@@ -322,11 +364,13 @@ class HttpRouter(object):
                 resp = yield _o(f)(req, **kwargs)
                 yield Return(resp)
             self.routes[method].append((pattern, replacement))
+            if add_head:
+                self.routes['HEAD'].append((pattern, replacement))
             return replacement
         return decorator
 
-    def get(self, pattern):
-        return self.mk_decorator('GET', pattern)
+    def get(self, pattern, add_head=True):
+        return self.mk_decorator('GET', pattern, add_head=add_head)
 
     def post(self, pattern):
         return self.mk_decorator('POST', pattern)
@@ -348,15 +392,30 @@ class HttpRouter(object):
 
     @_o
     def handle_request(self, req):
+        before = time.time()
+        resp = None
         for pattern, handler in self.routes[req.method]:
             match, kwargs = self.path_matches(urllib2.unquote(req.path),
                                               pattern)
             if match:
-                yield Return((yield handler(req, **kwargs)))
-        if self.handler:
+                resp = yield handler(req, **kwargs)
+        if self.handler and not resp:
             resp = yield self.handler(req)
-            yield Return(resp)
-        yield Return(404, {}, "")
+        if not resp:
+            resp = Return(404, {}, "")
+        after = time.time()
+
+        content_length = 0
+        if len(resp) > 2:
+            content_length = len(resp[2])
+
+        log.info("[%s] %s %s %s -> %s (%s bytes, %.0fms); %s",
+                 req.remote_ip,
+                 req.method, req.path, req.proto,
+                 resp[0], content_length, (after - before) * 1000,
+                 req.headers.get('user-agent'))
+
+        yield Return(resp)
 
 
 import monocle
